import processing.core.*; 
import java.util.TreeMap;
import java.util.ArrayList;
import java.io.Serializable;

class Song implements Serializable {
  String mp3File = null;
  String coverFile = null;

  float bpm = 0;
  float gap = 0;
  float fpb = 0;

  boolean relative = false;
  int resolution = 4;
  float offset = 0;

  //  TreeMap syllables = new TreeMap();
  TreeMap breaks = new TreeMap();
  public TreeMap lines = new TreeMap();

  void setKey(String newKey, String newValue) {
    //if (newKey.equals("RESOLUTION")) resolution = Integer.parseInt(newValue.replace(',','.'));
    if (newKey.equals("RELATIVE")) {
      if (newValue.equals("YES"))
        relative = true;
      else
        relative = false;
    }
    if (newKey.equals("GAP")) {
      gap = Float.valueOf(newValue.replace(',','.')).floatValue();
      gap = (gap / 1000) * 30; 
      offset = gap;
    }
    if (newKey.equals("BPM")) {
      bpm = Float.valueOf(newValue.replace(',','.')).floatValue();
      fpb = ((60 / bpm) / resolution) * 30;
    }
    if (newKey.equals("MP3")) mp3File = newValue;
    if (newKey.equals("COVER")) coverFile = newValue;
  }

  //  void addSyllable(int newFrame, String newSyllable) {
  //    Integer frameInteger = new Integer(newFrame);
  //    //println(frameInteger + " " + newSyllable);
  //    syllables.put(frameInteger, new Syllable(newSyllable,0,0));
  //  } 

  void addLine(Line newLine) {
    lines.put(newLine.getStartFrame(), newLine); 
  }

  void addBreak(int newStart, int newEnd) {
    if (relative) {
      offset = Float.valueOf(newEnd).floatValue();
    }
    breaks.put(newStart, new Break(newStart, newEnd)); 
  }

  Break getBreakAt(int currentFrame) {
    if (breaks.containsKey(currentFrame)) {
      return (Break)breaks.get(currentFrame);
    } 
    else {
      return null;
    }
  }

  //  Syllable getSyllableAt(int currentFrame) {
  //    Integer frameInteger = new Integer(currentFrame);
  //    if (syllables.containsKey(frameInteger)) {
  //      return (Syllable) syllables.get(frameInteger);
  //
  //    }
  //    else{
  //      return null;
  //    }
  //  }
  //
  //  Syllable getSyllableAfter(int currentFrame) {
  //    Integer frameInteger = new Integer(currentFrame);
  //    ArrayList keys = new ArrayList(syllables.keySet());
  //
  //    if (keys.indexOf(frameInteger) + 1 < keys.size()){
  //      int nextFrame = (Integer)keys.get((keys.indexOf(frameInteger))+1);
  //      return (Syllable) syllables.get(nextFrame);
  //
  //    } 
  //    else {
  //      return null;
  //    }
  //  }

  float getOffset() {
    return(offset); 
  }

  float getFPB() {
    return(fpb);
  }

  TreeMap getLines () {
    return(lines); 
  }

  boolean isRelative() {
    return(relative);
  }


}

class Break implements Serializable {
  int startFrame;
  int endFrame;

  Break(int newStart, int newEnd) {
    startFrame = newStart;
    endFrame = newEnd;
  }
}


class Line implements Serializable { 
  TreeMap words;
  String letters = "";
  int startFrame;
  int endFrame = 19800318;  
  float[] lineMatrix;
  float lineHeight = 480;
  int rotationFactor = 1;

  Line (int newStart) {
    words = new TreeMap();
    startFrame = newStart;
  }

  void addWord(Word newWord) {
    words.put(newWord.getStartFrame(), newWord); 
    letters = letters + newWord.getLetters();
  }

  void setMatrix(float[] newMatrix){
    lineMatrix = newMatrix;
  }

  float[] getMatrix() {
    return lineMatrix;
  }

  int getStartFrame() {
    return startFrame;
  }  

  int getEndFrame() {
    return endFrame;
  }  

  void setEndFrame(int newEndFrame) {
    endFrame = newEndFrame; 
  } 

  String getLetters() {
    return letters;
  }

  TreeMap getWords () {
    return(words); 
  }

  void setHeight(float newHeight) {
    lineHeight = newHeight; 
  }

  float getHeight() {
    return lineHeight; 
  }

  void setRotationFactor(int newRotationFactor) {
    rotationFactor = newRotationFactor;
  }

  int getRotationFactor(){
    return rotationFactor;
  }

}

class Word implements Serializable {
  TreeMap syllables;
  String letters = "";
  int startFrame;
  int endFrame = 19800318;  
  float[] wordMatrix;


  Word(int newStart) {
    syllables = new TreeMap();
    startFrame = newStart;
  }

  void addSyllable(Syllable newSyllable) {
    syllables.put(newSyllable.getStartFrame(), newSyllable); 
    letters = letters + newSyllable.getLetters();
  }

  void setMatrix(float[] newMatrix){
    wordMatrix = newMatrix;
  }

  float[] getMatrix() {
    return wordMatrix;
  }

  int getStartFrame() {
    return startFrame;
  }  

  int getEndFrame() {
    return endFrame;
  }  

  void setEndFrame(int newEndFrame) {
    endFrame = newEndFrame; 
  } 

  void setLetters(String newLetters) {
    letters = newLetters;
  }

  String getLetters() {
    return letters;
  }

  TreeMap getSyllables () {
    return(syllables); 
  }

}


class Syllable implements Serializable {
  String letters = "";
  int startFrame;
  int endFrame = 19800318;  
  float[] syllableMatrix;

  Syllable (int newStart, String newLetters) {
    letters = newLetters;
    startFrame = newStart;
  }

  //  Syllable (String newLetters, float newX, float newY) {
  //    letters = newLetters;
  //    drawX = newX;
  //    drawY = newY; 
  //  }

  void setLetters(String newLetters) {
    letters = newLetters;
  }  

  String getLetters() {
    return letters;
  }

  void setMatrix(float[] newMatrix){
    syllableMatrix = newMatrix;
  }

  float[] getMatrix() {
    return syllableMatrix;
  }

  int getStartFrame() {
    return startFrame;
  }  

  int getEndFrame() {
    return endFrame;
  }  

  void setEndFrame(int newEndFrame) {
    endFrame = newEndFrame; 
  }
}







